/*
 * Project name:
     Rotary click
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Revision History:
     - initial release (FJ);
 * Description:
     - This project is a simple demonstration of working with the Rotary click board.
     - Encoder pin A is connected to PD4, pin B is connected to PA7 and SW to PD2.
     - Use encoder to rotate LED's on Rotary click board, and SW to turn on one, two, four or eight LEDS.
 * Test configuration:
     MCU:             ATmega32
                      http://www.atmel.com/Images/doc2503.pdf
     Dev. Board:      EasyAVR7
                      http://www.mikroe.com/easyavr/
     Oscillator:      Oscillator Frequency 8.0000 MHz, 8.0000 MHz Crystal
     ext. modules:    Rotary click board click
                      http://www.mikroe.com/click/rotary/
     SW:              mikroC PRO for AVR
                      http://www.mikroe.com/mikroc/avr/
 * NOTES:
     - Place Rotary click board at the mikroBUS socket 1 on the EasyAVR v7.
     - Pull down PD2 on the EasyAVR v7 board.
     - Turn on SW5.4, SW5.5 and SW5.6 (SPI).
 */

sbit HC595_cs           at PORTA5_bit;
sbit ENCA               at PIND4_bit;
sbit ENCB               at PINA7_bit;
sbit ENC_SW             at PIND2_bit;
sbit HC595_cs_direction at DDA5_bit;
sbit ENCA_direction     at DDD4_bit;
sbit ENCB_direction     at DDA7_bit;
sbit ENC_SW_direction   at DDD2_bit;

unsigned int Led_State = 0x0001;
char ENCA_bit,ENCB_bit;
char ENC_Button_State = 0;

// ISR
void interrupt_ISR () org IVT_ADDR_INT0 {               // Interrupt routine
  SREG_I_bit = 0;                                       // Disable Interrupts
  ENC_Button_State++;                                   // set new ENC button state
  if (ENC_Button_State == 4)
    ENC_Button_State = 0;
    
  switch (ENC_Button_State){
      case 0 : Led_State = 0x0001;                      // One LED are ON
               break;
      case 1 : Led_State = 0x0101;                      // Two LEDs are ON
               break;
      case 2 : Led_State = 0x1111;                      // Four LEDs are ON
               break;
      case 3 : Led_State = 0x5555;                      // Eight LEDs are ON
               break;
      default :
               break;
    }
  SREG_I_bit = 1;                                       // Enable Interrupts
}

void HC595_Write(int value){
  char first_byte, second_byte;
  
  second_byte = value;
  value = value>>8;
  first_byte = value;

  SPI1_write(first_byte);
  SPI1_write(second_byte);
  
  HC595_cs = 0;
  asm{nop};
  asm{nop};
  asm{nop};
  HC595_cs = 1;
}

void main() {
  ENCA_direction     = 0;                               // Configure PD4 as input
  ENCB_direction     = 0;                               // Configure PA7 as input
  ENC_SW_direction   = 0;                               // Configure PD2 as input
  HC595_cs_direction = 1;                               // Configure PA5 as output
  
  GICR = 0x40;                                          // Set the Interrupts
  ISC01_bit = 1;
  ISC00_bit = 1;

  SREG_I_bit = 1;                                       // Enable Interrupts
  
  SPI1_Init();                                          // Initialize SPI1
  
  while(1) {
    if((ENCA_bit != ENCA)||(ENCB_bit != ENCB)){             // If logic state on pin A or pin B change
                                                            // Compare last logic state of pin B and new logic state on pin A
      if(ENCB_bit|ENCA==1){                                 // If ENCB_old OR ENCA_new is 1
        Led_State = (Led_State << 1) | (Led_State >> 15);
      }

      if(ENCA_bit|ENCB==1){                                 // If ENCA_old OR ENCB_new is 1
        Led_State = (Led_State >> 1) | (Led_State <<15);
      }

      ENCA_bit = ENCA;                                      // Set new value of ENCA_bit
      ENCB_bit = ENCB;                                      // Set new value of ENCB_bit
    }

  HC595_Write(Led_State);
  }
}